package com.bonus.material.notice.service;

import com.bonus.common.core.web.domain.AjaxResult;
import com.bonus.material.notice.entity.Notice;

import java.util.List;

/**
 * @Author：liang.chao
 * @Date：2024/12/20 - 19:22
 */
public interface NoticeService {
     /**
      * 查询公告列表
      * @param notice
      * @return
      */
     List<Notice> selectNoticeList(Notice notice);

     List<Notice> noticeList(Notice notice);

     List<Notice> messageList(Notice notice);

     /**
      * 新增通知公告
      * @param notice
      * @return
      */
     AjaxResult insertNotice(Notice notice);

     /**
      * 修改通知公告
      * @param notice
      * @return
      */
     AjaxResult updateNotice(Notice notice);

     /**
      * 删除通知公告
      * @param noticeIds
      * @return
      */
     AjaxResult deleteNoticeByIds(Long[] noticeIds);

}
